<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Repositories + data mapper all at once
// No interfaces
// mysql support only
namespace sgae3\Repository;

use Exception;

abstract class Repository
{
  
    protected $class;

    protected $dbTable;

    protected $db;

    public function getTable(){
        return $this->dbTable;
    }

    protected function fillInstance(object &$instance, array $row)
    {
        $all_properties = $instance->getFillable();
        array_unshift($all_properties, $instance->getIdentifier());

        foreach ($row as $field => $value) {
            if (! in_array($field, $all_properties)) {
                throw new Exception("Property '$field' for class '$this->class' does not exist");
            }
            $instance->set($field, $value);
        }
    }

    public function find(string $id)
    {
        $instance = new $this->class();

        $id = $this->db->real_escape_string($id);

        if (! $this->db->query("SELECT * FROM `" . $this->dbTable . "` WHERE `" . $instance->getIdentifier() . "` = '$id'")) {
            throw new Exception("Error loading '$this->class' data for $id: " . $this->db->error());
        }
        if ($this->db->num_results() > 1) {
            throw new Exception("More than one matching id '$id' for '$this->class'");
        }

        if ($this->db->num_results() == 0) {
            return null;
        }

        $row = $this->db->fetch_one();

        $this->fillInstance($instance, $row);
        return $instance;
    }

    public function findBy(string $varname, string $value)
    {

        $instance = new $this->class();

        $varname = $this->db->real_escape_string($varname);
        $value = $this->db->real_escape_string($value);

        if (! $this->db->query("SELECT * FROM `" . $this->dbTable . "` WHERE `" . $varname . "` = '$value'")) {
            throw new Exception("Error finding '$this->class' data by '$varname' from '$value'", E_USER_ERROR);
        }
        if ($this->db->num_results() > 1) {
            throw new Exception("More than one matching $varname '$value' for '$this->class'");
        }

        if ($this->db->num_results() == 0) {
            return null;
        }

        $row = $this->db->fetch_one();

        $this->fillInstance($instance, $row);
        return $instance;
    }
    
    public function findAllBy(string $varname, string $value)
    {
        $instance = new $this->class();
        
        $varname = $this->db->real_escape_string($varname);
        $value = $this->db->real_escape_string($value);
        
        if (! $this->db->query("SELECT * FROM `" . $this->dbTable . "` WHERE `" . $varname . "` = '$value'")) {
            throw new Exception("Error finding '$this->class' data by '$varname' from '$value'", E_USER_ERROR);
        }
        
        $output = array();
        foreach ($this->db->fetch_all() as $row) {
            $instance = new $this->class();
            $this->fillInstance($instance, $row);
            $output[] = $instance;
        }
        return $output;
    }

    public function save(object $instance)
    {
        if (! ($instance instanceof $this->class)) {
            trigger_error("Trying to save instance of " . get_class($instance) . " through repository for '$this->class'", E_USER_ERROR);
            throw new Exception("Repository error");
        }

        $sql_variable_names = "";
        $sql_variable_values = "";
        foreach ($instance->getFillable() as $value) {
            $sql_variable_names .= "`$value`,";
            $sql_variable_values .= "'" . $this->db->real_escape_string($instance->get($value)) . "',";
        }
        $sql_variable_names = rtrim($sql_variable_names, ",");
        $sql_variable_values = rtrim($sql_variable_values, ",");

        $sql_query = "INSERT INTO `" . $this->dbTable . "` ($sql_variable_names) VALUES ($sql_variable_values) ";
        if (! $this->db->query($sql_query)) {
            trigger_error("Error inserting '$this->class': " . $this->db->error(), E_USER_ERROR);
            throw new Exception("Database error");
        }

        return $this->db->insert_id();
    }

    public function update(object $instance)
    {
        if (! ($instance instanceof $this->class)) {
            trigger_error("Trying to update instance of " . get_class($instance) . " through repository for '$this->class'", E_USER_ERROR);
            throw new Exception("Repository error");
        }

        $sql_query = "UPDATE `$this->dbTable` SET";

        foreach ($instance->getFillable() as $value) {
            $sql_query .= " `$value` = '" . $this->db->real_escape_string($instance->get($value)) . "',";
        }
        $sql_query = rtrim($sql_query, ",");
        $sql_query .= "WHERE `" . $instance->getIdentifier() . "` = '" . $instance->getID() . "'";

        if (! $this->db->query($sql_query)) {
            trigger_error("Error updating '$this->class'", E_USER_ERROR);
            throw new Exception("Database error");
        }
    }

    public function delete(object $instance)
    {
        if (! ($instance instanceof $this->class)) {
            trigger_error("Trying to delete instance of " . get_class($instance) . " through repository for '$this->class'", E_USER_ERROR);
            throw new Exception("Repository error");
        }

        $sql_query = "DELETE FROM `" . $this->dbTable . "`
                WHERE `" . $instance->getIdentifier() . "` = '" . $instance->getID() . "'";

        if (! $this->db->query($sql_query)) {
            trigger_error("Error deleting '$this->class' with id '" . $instance->getID() . "'", E_USER_ERROR);
            throw new Exception("Database error");
        }
    }

    public function findAll()
    {
        $output = array();
        if (! $this->db->query("SELECT * FROM `$this->dbTable`")) {
            throw new Exception("Error searching all from " . $this->class);
        }

        $output = array();
        foreach ($this->db->fetch_all() as $row) {
            $instance = new $this->class();
            $this->fillInstance($instance, $row);
            $output[] = $instance;
        }
        return $output;
    }

    public function findAllNotIn(array $ids_list)
    {
        if (count($ids_list) == 0) {
            return $this->findAll();
        }

        $not_in = implode(",", $ids_list);

        $instance = new $this->class();

        $output = array();
        if (! $this->db->query("SELECT * FROM `$this->dbTable` 
            WHERE `" . $instance->getIdentifier() . "` NOT IN($not_in)")) {
            throw new Exception("Error searching all - not in - from " . $this->class);
        }

        $output = array();
        foreach ($this->db->fetch_all() as $row) {
            $instance = new $this->class();
            $this->fillInstance($instance, $row);
            $output[] = $instance;
        }
        return $output;
    }
}
?>
