<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Service;

use Exception;
use sgae3\Repository\AdminRepository;
use sgae3\Entity\Admin;

class AdminService
{

    public function isAdmin(string $username): bool
    {
        $adminRepository = new AdminRepository();
        try {
            if (is_null($adminRepository->findByUsername($username))) {
                return false;
            }
            return true;
        } catch (Exception $e) {
            trigger_error("Cannot determine if $username is admin: " . $e->getMessage());
            return false;
        }
    }

    public function getAdminUsernames(): array {
        $adminRepository = new AdminRepository();
        try {
            $admins = $adminRepository->findAll();
            $output = array();
            foreach ($admins as $admin) {
                $output[] = $admin->getUsername();
            }
            return $output;
        } catch (Exception $e) {
            trigger_error("Repository error browsing existing admins: " . $e->getMessage());
            throw new Exception("Cannot browse existing admins.");
        }
    }

    public function saveAdmins(array $admins): void
    {
        $cur_admins_usernames = $this->getAdminUsernames();

        $add_admins = array_diff($admins, $cur_admins_usernames);
        $del_admins = array_diff($cur_admins_usernames, $admins);

        //Insert new
        foreach ($add_admins as $admin) {
            $this->createAdmin($admin);
        }
        //Delete removed
        foreach ($del_admins as $admin) {
            $this->deleteAdmin($admin);
        }
    }

    public function createAdmin(string $username): void
    {
        $adminRepository = new AdminRepository();
        $admin = new Admin();
        $admin->setUsername($username);
        try {
            $adminRepository->save($admin);
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function deleteAdmin(string $username): void
    {
        $adminRepository = new AdminRepository();
        try {
            $admin = $adminRepository->findByUsername($username);
            $adminRepository->delete($admin);
        } catch (Exception $e) {
            throw $e;
        }
    }
}
