<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Service;

use Exception;
use sgae3\Repository\QueueAdminRepository;
use sgae3\Entity\QueueAdmin;
use sgae3\Repository\QueueRepository;

class QueueAdminService
{

    private $adminRepository;

    public function isQueueAdmin(string $username): bool
    {
        if (empty($this->getAdministeredQueues($username))) {
            return false;
        }
        return true;
    }

    public function getAdministeredQueues(string $username): array
    {
        $queueAdminRepository = new QueueAdminRepository();
        return $queueAdminRepository->findByUsername($username);
    }

    //Includes all queues, even those without admins
    public function getFullQueueAdmins(): array
    {
        $queueAdminRepository = new QueueAdminRepository();
        $queueService = new QueueService();

        try {
            $queues = $queueService->getAllQueues();
            $queue_admins = $queueAdminRepository->findAll();
        } catch (Exception $e) {
            throw $e;
        }

        $output = array();
        foreach ($queues as $queue) {
            $output[$queue->getID()] = array();
        }
        foreach ($queue_admins as $queue_admin) {
            $output[$queue_admin->getQueueID()][] = $queue_admin->getUsername();
        }

        return $output;
    }

    public function saveQueueAdmins(string $queue_id, array $p_queue_admins)
    {

        $queueAdminRepository = new QueueAdminRepository();
        try {
            $queue_admins = $queueAdminRepository->findByQueue($queue_id);
        } catch (Exception $e) {
            throw $e;
        }

        $cur_queue_admins = array();
        foreach ($queue_admins as $queue_admin) {
            $cur_queue_admins[] = $queue_admin->getUsername();
        }

        $add_admins = array_diff($p_queue_admins, $cur_queue_admins);
        $del_admins = array_diff($cur_queue_admins, $p_queue_admins);

        //Insert new
        foreach ($add_admins as $admin) {
            $this->createQueueAdmin($queue_id, $admin);
        }
        //Delete removed
        foreach ($del_admins as $admin) {
            $this->deleteQueueAdmin($queue_id, $admin);
        }
    }

    function createQueueAdmin(string $queue_id, string $username): void
    {
        if ($username == "") return;

        $queueAdmin = new QueueAdmin();
        $queueAdmin->setQueueID($queue_id);
        $queueAdmin->setUsername($username);
        $queueAdminRepository = new QueueAdminRepository;
        try {
            $queueAdminRepository->save($queueAdmin);
        } catch (Exception $e) {
            throw $e;
        }
    }

    function deleteQueueAdmin(string $queue_id, string $username): void
    {
        $queueAdminRepository = new QueueAdminRepository();
        try {
            $queueAdmin = $queueAdminRepository->findByQueueAndUsername($queue_id, $username);
            $queueAdminRepository->delete($queueAdmin);
        } catch (Exception $e) {
            throw $e;
        }
    }
}
