<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Service;

use Exception;
use sgae3\Repository\QueueRepository;
use sgae3\Entity\Queue;

// use sgae3\Entity\Queue;
class QueueService
{

    public function getAllQueues()
    {
        $queueRepository = new QueueRepository();
        return $queueRepository->findAll();
    }

    public function getEnabledQueues()
    {
        $queueRepository = new QueueRepository();
        return $queueRepository->findEnabled();
    }

    public function getAdministeredQueues(string $username){
        $qaS = new QueueAdminService();
        $queueRepository = new QueueRepository();
        $output = array();
        foreach ($qaS->getAdministeredQueues($username) as $admin_queue){
            $output[] = $queueRepository->find($admin_queue->getQueueID());
        }
        return $output;
    }

    public function userIsQueueAdmin(string $username, string $queue_name) : bool {
        foreach($this->getAdministeredQueues($username) as $queue) {                
            if($queue->getQueueName() == $queue_name){
                return true;
            }
        }
        return false;
    }

    public function newQueue(string $queue_name)
    {
        $queue = new Queue();
        $queueRepository = new QueueRepository();
        $queue->setQueueName($queue_name);
        $queue->setLastCheckedDate(0);
        $queue->setEnabledQueue(1);
        try {
            $queueRepository->save($queue);
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function toogleEnabled(string $queue_id)
    {
        $queueRepository = new QueueRepository();
        try {
            $queue = $queueRepository->find($queue_id);
            $enabled = $queue->getEnabledQueue();
            if ($enabled == 1) {
                $queue->setEnabledQueue(0);
            }else{
                $queue->setEnabledQueue(1);
            }
            $queueRepository->update($queue);
        } catch (Exception $e) {
            throw $e;
        }
        
    }

    public function reset(string $queue_id)
    {
        $queueRepository = new QueueRepository();
        try {
            $queue = $queueRepository->find($queue_id);
            $queue->setLastCheckedDate(0);
            $queueRepository->update($queue);
        } catch (Exception $e) {
            throw $e;
        }
        
    }

    public function update(object $queue){
        $queueRepository = new QueueRepository();
        try {
            $queueRepository->update($queue);
        } catch (Exception $e) {
            throw $e;
        }
    }
}

?>
