/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

var countdown;

$(function () {

	$("#nav_admin").addClass("active");

	//Remember last visited tab
	$("#tabSelector a").click(function (e) {
		localStorage.setItem('activeTab', $(e.target).attr('href'));
	});
	var activeTab = localStorage.getItem('activeTab');
	if (activeTab) {
		$('#tabSelector a[href="' + activeTab + '"]').tab('show');
	}

	//Prevent form submit on intro
	$('form input').keydown(function (e) {
		if (e.keyCode == 13) {
			e.preventDefault();
			return false;
		}
	});

	$("#newQueue").click(registerQueue);
	$('button[id^="saveQueueAdmins"]').click(function(){
		saveQueueAdmins(this.id.split("_")[1])
	})
	$('button[id^="resetQueue"]').click(function(){
		resetQueue(this.id.split("_")[1])
	})
	$("#saveAdmins").click(saveAdmins);
	$("#refreshCron").click(refreshCronStatus);
	$('#runCron').click(runCron);
	$("#newUser").click(registerUser);
	$('button[id^="changePasswd"]').click(function(){
		changePassword(this.id.split("_")[1])
	})

	countdown = setInterval('statusCountdown()', 1000);
	statusCountdown();
	refreshCronStatus()
})

function resetQueue(id) {
	$.ajax({
		method: "POST",
		url: "admin/queue/reset",
		data: {
			queueId: id,
		},
		success: [ajax_success, () => { $("#lastCheckedDate" + id).text("reseted") }],
		error: ajax_error
	});

}

function registerQueue() {
	var queueName = $("#newQueueName").val();

	if ($.trim(queueName) == "") {
		$("#newQueueName").addClass('is-invalid');
		return false;
	} else {
		$("#newQueueName").removeClass('is-invalid');
	}

	$.ajax({
		method: "POST",
		url: "admin/queue/new",
		data: {
			queueName: queueName
		},
		success: [ajax_success, () => { setTimeout("location.reload()", 1000) }],
		error: ajax_error
	});

	return false;
}

function registerUser() {
	var username = $("#newUsername").val();
	if ($.trim(username) == "") {
		$("#newUsername").addClass('is-invalid');
		return false;
	} else {
		$("#newUsername").removeClass('is-invalid');
	}

	var newUserPassword = $("#newUserPassword").val();
	if ($.trim(newUserPassword) == "") {
		$("#newUserPassword").addClass('is-invalid');
		return false;
	} else {
		$("#newUserPassword").removeClass('is-invalid');
	}

	$.ajax({
		method: "POST",
		url: "admin/user/new",
		data: {
			username: username,
			password: newUserPassword
		},
		success: [ajax_success, () => { setTimeout("location.reload()", 1000) }],
		error: ajax_error
	});

	return false;
}

function saveQueueAdmins(queue_id) {
	$.ajax({
		method: "POST",
		url: "admin/saveQueueAdmins",
		data: {
			queue_id: queue_id,
			admins: $("#queueAdmins" + queue_id).val()
		},
		success: ajax_success,
		error: ajax_error
	});
}

function toogleEnabled(qid) {
	$.ajax({
		method: "POST",
		url: "admin/queue/toogleEnabled",
		data: {
			queueId: qid
		},
		success: ajax_success,
		error: ajax_error
	});
}

function saveAdmins() {
	$.ajax({
		method: "POST",
		url: "admin/saveAdmins",
		data: {
			admins: $('#adminsUsernames').val()
		},
		success: ajax_success,
		error: ajax_error
	});
	return false;
}

function statusCountdown() {
	count = $('#countdown').html();
	if (count <= "1") {
		count = 59;
		getCronStatus();
	} else count--;
	$('#countdown').text(count);
}

function refreshCronStatus() {
	getCronStatus();
	clearInterval(countdown);
	$('#countdown').text("59");
	countdown = setInterval('statusCountdown()', 1000);
}

function runCron() {
	$.ajax({
		method: "POST",
		url: "admin/runCron",
		success: [ajax_success, () => { $("#status").text("running") }],
		error: ajax_error
	});
}

function getCronStatus() {
	$.ajax({
		method: "POST",
		url: "admin/getCronStatus",
		success: function (data) {
			try {
				var json = JSON.parse(data);
				if (json['status'] == "error") {
					display_error(json['error'])
				} else {
					$("#status").text(json['msg']);
				}
			} catch (e) {
				display_error("An error has occurred")
				console.log(e)
			}
		},
		error: ajax_error
	});
}

function deleteUser(e, id) {
	$.ajax({
		method: "POST",
		url: "admin/user/delete",
		data: {
			user_id: id
		},
		success: [ajax_success, () => { setTimeout("location.reload()", 1000) }],
		error: ajax_error
	});

	e.preventDefault();

}

function changePassword(id) {
	var newUserPassword = $("#userNewPassword_"+id).val();
	if ($.trim(newUserPassword) == "") {
		$("#userNewPassword_"+id).addClass('is-invalid');
		return false;
	} else {
		$("#userNewPassword_"+id).removeClass('is-invalid');
	}

	$.ajax({
		method: "POST",
		url: "admin/user/changePassword",
		data: {
			user_id: id,
			password: newUserPassword
		},
		success: [ajax_success],
		error: ajax_error
	});

	return false;
}