/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Charts config
const user_charts = [
    { id: "user_exec_time", data: "user_exec_time_data", func: "stacked_bar" },
    { id: "user_queue_usage", data: "user_queue_usage_data", func: "pie" },
    { id: "user_exec_jobs", data: "user_exec_jobs_data", func: "pie" },
    { id: "user_mem_usage", data: "user_mem_usage_data", func: "stacked_bar" },
    { id: "user_slots_usage", data: "user_slots_usage_data", func: "bar" },
    { id: "user_topten_node_usage", data: "user_topten_node_usage_data", func: "top_ten" },
    { id: "user_topten_execution_time_abs", data: "user_topten_execution_time_abs_data", func: "top_ten" }
]

// Also save charts in global objects for update
var user_exec_time, user_queue_usage, user_exec_jobs, user_mem_usage, user_slots_usage, user_topten_node_usage, user_topten_execution_time_abs

function prepare_data_for_user_graphs(json_data) {

    $("#user_running_jobs").text(json_data.user_running_jobs);
    $("#user_queued_jobs").text(json_data.user_queued_jobs);
    $("#user_cluster_total_jobs").text(json_data.user_cluster_total_jobs);

    graph_footer("user_exec_time_footer", 
        parseFloat(json_data.jobs_execution_time_longest / 60).toFixed(2)+" min", 
        parseFloat(json_data.jobs_execution_time_shortest / 60).toFixed(2)+" min", 
        parseFloat(json_data.jobs_execution_time_average / 60).toFixed(2)+" min");
    
    graph_footer("user_mem_usage_footer", 
        parseFloat(json_data.jobs_memory_maximum / 1024).toFixed(2)+"MB", 
        parseFloat(json_data.jobs_memory_minimum / 1024).toFixed(2)+"MB", 
        parseFloat(json_data.jobs_memory_average / 1024).toFixed(2)+"MB");

    pie_graph_footer("user_exec_jobs_footer", json_data.jobs_failure, "failed", json_data.jobs_success, "success")
    pie_graph_footer("user_queue_usage_footer", json_data.owner_queue_usage, "you", json_data.non_owner_queue_usage, "other")

    

    user_exec_time_data.datasets[0].data[0] = json_data.jobs_execution_time_1d;
    user_exec_time_data.datasets[1].data[0] = json_data.jobs_execution_time_7d;
    user_exec_time_data.datasets[2].data[0] = json_data.jobs_execution_time_31d;
    user_exec_time_data.datasets[3].data[0] = json_data.jobs_execution_time_xxd;

    user_queue_usage_data.datasets[0].data = [
        json_data.owner_queue_usage, json_data.non_owner_queue_usage
    ]

    user_exec_jobs_data.datasets[0].data = [
        json_data.jobs_failure, json_data.jobs_success
    ]

    user_mem_usage_data.datasets[0].data[0] = json_data.jobs_memory_1gb;
    user_mem_usage_data.datasets[1].data[0] = json_data.jobs_memory_4gb;
    user_mem_usage_data.datasets[2].data[0] = json_data.jobs_memory_8gb;
    user_mem_usage_data.datasets[3].data[0] = json_data.jobs_memory_xxgb;

    user_slots_usage_data.datasets[0].data = [
        json_data.Slots_1,
        json_data.Slots_2,
        json_data.Slots_4,
        json_data.Slots_8,
        json_data.Slots_16,
        json_data.Slots_32,
        json_data.Slots_64,
        json_data.Slots_128,
        json_data.Slots_x
    ]

    user_topten_node_usage_data.labels = json_data.user_topten_node_usage.labels;
    user_topten_node_usage_data.datasets[0].data = json_data.user_topten_node_usage.values;

    user_topten_execution_time_abs_data.datasets[0].data = json_data.user_topten_execution_time_abs.values;

}

var user_exec_time_data = {
    labels: ['Number of jobs'],
    datasets: [{
        label: '< 1 day',
        data: [],
        backgroundColor: backgroundColor[0],
        borderColor: borderColor[0],
        borderWidth: 1
    }, {
        label: '1 day - 1 week',
        data: [],
        backgroundColor: backgroundColor[1],
        borderColor: borderColor[1],
        borderWidth: 1
    }, {
        label: '1 week - 1 month',
        data: [],
        backgroundColor: backgroundColor[2],
        borderColor: borderColor[2],
        borderWidth: 1
    }, {
        label: '> 1 month',
        data: [],
        backgroundColor: backgroundColor[3],
        borderColor: borderColor[3],
        borderWidth: 1
    }]
}

var user_queue_usage_data = {
    labels: ['you', 'other'],
    datasets: [{
        label: 'Queue usage',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var user_exec_jobs_data = {
    labels: ['failed', 'completed'],
    datasets: [{
        label: 'Executed Jobs',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var user_mem_usage_data = {
    labels: ['Number of jobs'],
    datasets: [{
        label: '< 1GB',
        data: [],
        backgroundColor: backgroundColor[0],
        borderColor: borderColor[0],
        borderWidth: 1
    }, {
        label: '1GB - 4 GB',
        data: [],
        backgroundColor: backgroundColor[1],
        borderColor: borderColor[1],
        borderWidth: 1
    }, {
        label: '4GB - 8GB',
        data: [],
        backgroundColor: backgroundColor[2],
        borderColor: borderColor[2],
        borderWidth: 1
    }, {
        label: '> 8GB ',
        data: [],
        backgroundColor: backgroundColor[3],
        borderColor: borderColor[3],
        borderWidth: 1
    }]
}

var user_slots_usage_data = {
    labels: ['1', '2', '3-4', '5-8', '9-16', '17-32', '33-64', '65-128', '+128'],
    datasets: [{
        label: '# of Slots',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var user_topten_node_usage_data = {
    labels: [],
    datasets: [{
        label: '# of jobs',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var user_topten_execution_time_abs_data = {
    labels: ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10'],
    datasets: [{
        label: 'Time (hours)',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}